'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _deepEqual = require('deep-equal');

var _deepEqual2 = _interopRequireDefault(_deepEqual);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var methodProperties = function methodProperties() {
	var props = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
	var state = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];


	var comparisonCache = {};

	var compare = function compare(value1, value2) {
		var values = Object.assign({}, props, state);
		if (!Object.keys(values).length) {
			return false;
		}
		var id = value1 + '-' + value2;
		if (comparisonCache.hasOwnProperty(id)) {
			return comparisonCache[id];
		} else {
			var result = !(0, _deepEqual2.default)(values[value1], values[value2]);
			comparisonCache[id] = result;
			return result;
		}
	};

	return {
		constructorMethod: {
			args: ['props'],
			state: [],
			props: [{
				name: 'props',
				value: props.initialProps,
				isChanged: true
			}],
			terminal: false,
			description: ''
		},
		componentWillMount: {
			args: [],
			state: [{
				name: 'this.state',
				value: state.initialState,
				isChanged: true
			}],
			props: [{
				name: 'this.props',
				value: props.initialProps,
				isChanged: false
			}],
			terminal: false,
			description: ''
		},
		componentDidMount: {
			args: [],
			state: [{
				name: 'this.state',
				value: state.mountedState,
				isChanged: false
			}],
			props: [{
				name: 'this.props',
				value: props.initialProps,
				isChanged: false
			}],
			terminal: true,
			description: 'Avoid calling setState here because it will trigger an extra render.'
		},
		componentWillReceiveProps: {
			args: ['nextProps'],
			state: [{
				name: 'this.state',
				value: state.rerenderedInitialState,
				isChanged: compare('mountedState', 'rerenderedInitialState')
			}],
			props: [{
				name: 'this.props',
				value: props.renderedInitialProps,
				isChanged: compare('initialProps', 'renderedInitialProps')
			}, {
				name: 'nextProps',
				value: props.newProps,
				isChanged: compare('renderedInitialProps', 'newProps')
			}],
			propsNotEqual: compare('renderedInitialProps', 'newProps'),
			terminal: false
		},
		shouldComponentUpdate: {
			args: ['nextProps', 'nextState'],
			state: [{
				name: 'this.state',
				value: state.rerenderedInitialStateAfterProps,
				isChanged: compare('mountedState', 'rerenderedInitialStateAfterProps')
			}, {
				name: 'nextState',
				value: state.rerenderedNewState,
				isChanged: compare('rerenderedInitialStateAfterProps', 'rerenderedNewState')
			}],
			stateNotEqual: compare('rerenderedInitialStateAfterProps', 'rerenderedNewState'),
			props: [{
				name: 'this.props',
				value: props.renderedInitialProps,
				isChanged: false
			}, {
				name: 'nextProps',
				value: props.newProps,
				isChanged: false
			}],
			propsNotEqual: compare('renderedInitialProps', 'newProps'),
			terminal: false
		},
		componentWillUpdate: {
			args: ['nextProps', 'nextState'],
			state: [{
				name: 'this.state',
				value: state.rerenderedInitialStateAfterProps,
				isChanged: false
			}, {
				name: 'nextState',
				value: state.rerenderedNewState,
				isChanged: compare('rerenderedInitialStateAfterProps', 'rerenderedNewState')
			}],
			stateNotEqual: compare('rerenderedInitialStateAfterProps', 'rerenderedNewState'),
			props: [{
				name: 'this.props',
				value: props.renderedInitialProps,
				isChanged: false
			}, {
				name: 'nextProps',
				value: props.newProps,
				isChanged: false
			}],
			propsNotEqual: compare('renderedInitialProps', 'newProps'),
			terminal: false
		},
		render: {
			args: [],
			state: [{
				name: 'this.state',
				// Values depends on whether this is an initial render or a rerender
				value: state.rerenderedNewState !== null ? state.rerenderedNewState : state.mountedState,
				// Value is never changed if render is part of a rerender cycle but can change when mounting
				isChanged: compare(state.rerenderedInitialStateAfterProps !== null ? 'rerenderedInitialStateAfterProps' : 'rerenderedInitialStateAfterProps', state.rerenderedNewState !== null ? 'rerenderedNewState' : 'mountedState')
			}],
			props: [{
				name: 'this.props',
				value: props.newProps !== null ? props.newProps : props.initialProps,
				isChanged: false
			}],
			terminal: false
		},
		componentDidUpdate: {
			args: ['prevProps', 'prevState'],
			state: [{
				name: 'prevState',
				value: state.rerenderedInitialStateAfterProps,
				isChanged: false
			}, {
				name: 'this.state',
				value: state.rerenderedNewState,
				isChanged: false
			}],
			stateNotEqual: compare('rerenderedInitialStateAfterProps', 'rerenderedNewState'),
			props: [{
				name: 'prevProps',
				value: props.renderedInitialProps,
				isChanged: false
			}, {
				name: 'this.props',
				value: props.newProps,
				isChanged: false
			}],
			propsNotEqual: compare('renderedInitialProps', 'newProps'),
			terminal: true,
			description: 'Avoid calling setState here because it will trigger an extra render.'
		},
		componentWillUnmount: {
			args: [],
			state: [{
				name: 'this.state',
				value: state.rerenderedNewState,
				// TODO Technically this could change before component is unmounted
				isChanged: false
			}],
			props: [{
				name: 'this.props',
				value: props.newProps,
				isChanged: false
			}],
			terminal: true
		}
	};
};

exports.default = methodProperties;
module.exports = exports['default'];