'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _deepEqual = require('deep-equal');

var _deepEqual2 = _interopRequireDefault(_deepEqual);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var methodProperties = function methodProperties() {
	var props = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
	var state = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];


	var comparisonCache = {};

	var compare = function compare(value1, value2) {
		var values = Object.assign({}, props, state);
		if (!Object.keys(values).length) {
			return false;
		}
		var id = value1 + '-' + value2;
		if (comparisonCache.hasOwnProperty(id)) {
			return comparisonCache[id];
		} else {
			var result = !(0, _deepEqual2.default)(values[value1], values[value2]);
			comparisonCache[id] = result;
			return result;
		}
	};

	return {
		constructorMethod: {
			args: ['props'],
			displayNames: {
				props: ['props']
			},
			lifecycleLocation: 'mounting',
			save: ['props'],
			terminal: false
		},
		componentWillMount: {
			args: [],
			displayNames: {
				props: ['this.props'],
				state: ['this.state']
			},
			lifecycleLocation: 'mounting',
			save: ['state'],
			terminal: false
		},
		componentDidMount: {
			args: [],
			displayNames: {
				props: ['this.props'],
				state: ['this.state']
			},
			terminal: true,
			description: 'Avoid calling setState here because it will trigger an extra render.'
		},
		componentWillReceiveProps: {
			args: ['nextProps'],
			displayNames: {
				props: ['this.props', 'nextProps'],
				state: ['this.state']
			},
			lifecycleLocation: 'updating',
			save: ['props', 'state'],
			terminal: false
		},
		shouldComponentUpdate: {
			args: ['nextProps', 'nextState'],
			displayNames: {
				props: ['this.props', 'nextProps'],
				state: ['this.state', 'nextState']
			},
			lifecycleLocation: 'updating',
			save: ['state'],
			terminal: false
		},
		componentWillUpdate: {
			args: ['nextProps', 'nextState'],
			displayNames: {
				props: ['this.props', 'nextProps'],
				state: ['this.state', 'nextState']
			},
			terminal: false
		},
		render: {
			args: [],
			displayNames: {
				props: ['this.props'],
				state: ['this.state']
			},
			lifecycleLocation: 'dynamic',
			save: ['state'],
			terminal: false
		},
		componentDidUpdate: {
			args: ['prevProps', 'prevState'],
			displayNames: {
				props: ['prevProps', 'this.props'],
				state: ['prevState', 'this.state']
			},
			terminal: true,
			description: 'Avoid calling setState here because it will trigger an extra render.'
		},
		componentWillUnmount: {
			args: [],
			displayNames: {
				props: ['this.props'],
				state: ['this.state']
			},
			lifecycleLocation: 'unmounting',
			save: ['state'],
			terminal: true
		}
	};
};

exports.default = methodProperties;
module.exports = exports['default'];