'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

var _reactRedux = require('react-redux');

var _redux = require('redux');

var _radium = require('radium');

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _PopoutWindow = require('./components/PopoutWindow.js');

var _PopoutWindow2 = _interopRequireDefault(_PopoutWindow);

var _reducers = require('./store/reducers');

var _reducers2 = _interopRequireDefault(_reducers);

var _lifecycleConfig = require('./store/lifecycleConfig');

var _lifecycleConfig2 = _interopRequireDefault(_lifecycleConfig);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var consoleWindow = null;

var initialStore = _immutable2.default.fromJS({ entries: {} }).toMap();
var store = (0, _redux.createStore)(_reducers2.default, initialStore);

var log = {

	isTimerOn: false,

	add: function add(action) {
		var key = action.key;
		var name = action.name;

		var id = this.makeId(key, name);
		store.dispatch({
			type: 'ADD',
			key: key,
			name: name,
			id: id,
			displayName: this.makeDisplayName(key, name)
		});
		return id;
	},
	updateStore: function updateStore(action) {
		store.dispatch(action);
	},
	getFromStore: function getFromStore(keyPath) {
		return store.getState().getIn(keyPath);
	},
	makeId: function makeId(key, name) {
		return key ? name + '-' + key : name;
	},
	makeDisplayName: function makeDisplayName(key, name) {
		var formattedName = this.removeComponentWrapperNames(name);
		return key ? formattedName + ' (' + key + ')' : formattedName;
	},
	removeComponentWrapperNames: function removeComponentWrapperNames(name) {
		var wrapperPattern = /[^\(]+\(([^\)]+)\)/;
		while (wrapperPattern.test(name)) {
			name = name.replace(/[^\(]+\(([^\)]+)\)/, '$1');
		}
		return name;
	},
	addCalculatedValues: function addCalculatedValues() {
		var entries = store.getState().get('entries');
		return _lifecycleConfig2.default.addRemainingPropertiesToAllEntries(entries);
	},


	// The timer returns control to the wrapped application while
	// Visible React processes a batch of updates and does it's own rendering.
	// This is necessary for performance.
	updateWindow: function updateWindow(lifecycleId) {
		var _this = this;

		if (consoleWindow === null || consoleWindow.closed) {
			return;
		}
		if (this.isTimerOn) {
			return;
		}
		this.isTimerOn = true;
		var self = this;
		setTimeout(function () {
			var entries = _this.addCalculatedValues();
			var container = consoleWindow.document.getElementById('visible-react');
			_reactDom2.default.render(_react2.default.createElement(
				_reactRedux.Provider,
				{ store: store },
				_react2.default.createElement(
					_radium.StyleRoot,
					null,
					_react2.default.createElement(_PopoutWindow2.default, {
						entries: entries
					})
				)
			), container);
			_this.isTimerOn = false;
		}, 0);
	},
	getWindow: function getWindow() {
		if (window && consoleWindow === null || consoleWindow.closed) {
			consoleWindow = window.open('', 'console', "width=1350,height=900,resizable,scrollbars=yes,status=1");
			if (!consoleWindow) {
				alert('You must disable your popup blocker to use the Visible React Console.');
			}
			consoleWindow.document.title = 'Visible React';
			var container = consoleWindow.document.createElement('div');
			container.id = 'visible-react';
			consoleWindow.document.body.appendChild(container);
			consoleWindow.focus();
			//this.updateWindow();
			window.onbeforeunload = function () {
				consoleWindow.close();
			};
		}
		return consoleWindow;
	}
};

exports.default = log;
module.exports = exports['default'];