'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _deepEqual = require('deep-equal');

var _deepEqual2 = _interopRequireDefault(_deepEqual);

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var lifecycleConfig = {

	isPartnerChanged: {
		props: null,
		state: null
	},

	addRemainingPropertiesToAllEntries: function addRemainingPropertiesToAllEntries(entries) {
		var _this = this;

		return entries.map(function (entry) {
			return _this.addRemainingPropertiesToEntry(entry);
		});
	},
	addRemainingPropertiesToEntry: function addRemainingPropertiesToEntry(entry) {
		var _this2 = this;

		this.isPartnerChanged = {
			props: null,
			state: null
		};
		var isChanged = {
			props: false,
			state: false
		};

		var remainingPropertiesForEntry = this.properties;
		var newMethods = entry.get('methods').map(function (method, methodName) {
			// TODO refactor repeated code
			var remainingPropertiesForMethod = remainingPropertiesForEntry[methodName];

			var propValues = method.get('props');
			remainingPropertiesForMethod.props.values = propValues;
			if (remainingPropertiesForMethod.props.hasOwnProperty('arePartnersDifferent')) {
				remainingPropertiesForMethod.props.arePartnersDifferent = _this2.arePartnerPropsOrStatesDifferent(propValues, 'props');
			}
			if (remainingPropertiesForMethod.props.hasOwnProperty('isChanged')) {
				var isPropsChanged = _this2.isPropsOrStateChanged(propValues, methodName, 'props', entry);
				remainingPropertiesForMethod.props.isChanged = isPropsChanged;
				if (isPropsChanged && !isChanged.props) {
					isChanged.props = true;
				}
			}

			var stateValues = method.get('state');
			remainingPropertiesForMethod.state.values = stateValues;
			if (remainingPropertiesForMethod.state.hasOwnProperty('arePartnersDifferent')) {
				remainingPropertiesForMethod.state.arePartnersDifferent = _this2.arePartnerPropsOrStatesDifferent(stateValues, 'state');
			}
			if (remainingPropertiesForMethod.state.hasOwnProperty('isChanged')) {
				var isStateChanged = _this2.isPropsOrStateChanged(stateValues, methodName, 'state', entry);
				remainingPropertiesForMethod.state.isChanged = isStateChanged;
				if (isStateChanged && !isChanged.state) {
					isChanged.state = true;
				}
			}
			return method.merge(_immutable2.default.Map(remainingPropertiesForMethod));
		});

		return entry.set('methods', newMethods).set('isChanged', isChanged);
	},
	arePartnerPropsOrStatesDifferent: function arePartnerPropsOrStatesDifferent(immutableItems, type) {
		if (immutableItems.size < 2) {
			return false;
		}
		// Immutability doesn't help with comparison here because the two source objects
		// were different; convert to JS for faster comparison
		var items = immutableItems.toJS();
		if (this.isPartnerChanged[type] === null) {
			this.isPartnerChanged[type] = !(0, _deepEqual2.default)(items[0], items[1], { strict: true });
		}
		return this.isPartnerChanged[type];
	},
	isPropsOrStateChanged: function isPropsOrStateChanged(values, methodName, type, entry) {
		var method = entry.getIn(['methods', methodName]);
		var lifecycleLocation = method.get('lifecycleLocation');
		var isCalled = method.get('called');
		if (!isCalled) {
			return false;
		}
		if (lifecycleLocation === 'mounting' && !values.first()) {
			return false;
		}
		if (methodName === 'constructorMethod' || methodName === 'componentWillMount') {
			return true;
		}
		var precedingValue = void 0;
		if (methodName === 'render') {
			if (lifecycleLocation === 'mounting') {
				precedingValue = entry.getIn(['methods', 'componentWillMount', type, '0']);
			} else {
				return false;
			}
		} else if (methodName === 'componentWillReceiveProps' && type === 'props') {
			precedingValue = values.first();
		} else if (methodName === 'shouldComponentUpdate' && type === 'state') {
			precedingValue = values.first();
		} else {
			return false;
		}
		// Only the second of two parallel values will be marked changed;
		// for example, nextProps
		var value1ToCompare = values.last() ? values.last().toJS() : '';
		var value2ToCompare = precedingValue ? precedingValue.toJS() : '';
		return !(0, _deepEqual2.default)(value1ToCompare, value2ToCompare, { strict: true });
	},


	get properties() {
		return {
			constructorMethod: {
				args: ['props'],
				props: {
					names: ['props'],
					values: [],
					isChanged: false
				},
				state: {
					names: [],
					values: []
				},
				terminal: false
			},
			componentWillMount: {
				args: [],
				props: {
					names: ['this.props'],
					values: []
				},
				state: {
					names: ['this.state'],
					values: [],
					isChanged: false
				},
				terminal: false
			},
			render: {
				args: [],
				props: {
					names: ['this.props'],
					values: []
				},
				state: {
					names: ['this.state'],
					values: [],
					isChanged: false
				},
				terminal: false
			},
			componentDidMount: {
				args: [],
				props: {
					names: ['this.props'],
					values: []
				},
				state: {
					names: ['this.state'],
					values: []
				},
				terminal: true,
				description: 'Avoid calling setState here because it will trigger an extra render.'
			},
			componentWillReceiveProps: {
				args: ['nextProps'],
				props: {
					names: ['this.props', 'nextProps'],
					values: [],
					arePartnersDifferent: false,
					isChanged: false
				},
				state: {
					names: ['this.state'],
					values: []
				},
				terminal: false
			},
			shouldComponentUpdate: {
				args: ['nextProps', 'nextState'],
				props: {
					names: ['this.props', 'nextProps'],
					values: [],
					arePartnersDifferent: false
				},
				state: {
					names: ['this.state', 'nextState'],
					values: [],
					isChanged: false,
					arePartnersDifferent: false
				},
				terminal: false
			},
			componentWillUpdate: {
				args: ['nextProps', 'nextState'],
				props: {
					names: ['this.props', 'nextProps'],
					values: [],
					arePartnersDifferent: false
				},
				state: {
					names: ['this.state', 'nextState'],
					values: [],
					arePartnersDifferent: false
				},
				terminal: false
			},
			componentDidUpdate: {
				args: ['prevProps', 'prevState'],
				props: {
					names: ['prevProps', 'this.props'],
					values: [],
					arePartnersDifferent: false
				},
				state: {
					names: ['prevState', 'this.state'],
					values: [],
					arePartnersDifferent: false
				},
				terminal: true,
				description: 'Avoid calling setState here because it will trigger an extra render.'
			},
			componentWillUnmount: {
				args: [],
				props: {
					names: ['this.props'],
					values: []
				},
				state: {
					names: ['this.state'],
					values: []
				},
				terminal: true
			}
		};
	},

	getDisplayNames: function getDisplayNames(methodName, type) {
		return this.properties[methodName][type];
	},


	get methodNames() {
		return Object.keys(this.properties);
	}

};

exports.default = lifecycleConfig;
module.exports = exports['default'];