'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _root = require('../root.js');

var _root2 = _interopRequireDefault(_root);

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var entries = function entries(state, action) {

	switch (action.type) {
		// TODO should this be immutable?
		case 'ADD':
			var key = action.key;
			var id = action.id;
			var name = action.name;
			var displayName = action.displayName;

			var entry = {
				id: id,
				displayName: displayName,
				name: name,
				key: key,
				renderCount: 0,
				unnecessaryUpdatesPrevented: 0,
				isMounted: false,
				isChanged: {
					props: false,
					state: false
				},
				lifecycleLocation: '',
				methods: initMethods()
			};
			return state.setIn(['entries', id], _immutable2.default.fromJS(entry).toMap());
		case 'UPDATE_METHOD':
			return state.updateIn(['entries', action.entryId, 'methods', action.methodName], function (oldValue) {
				return oldValue.mergeDeep(_immutable2.default.fromJS(action.value));
			});
		case 'UPDATE_VALUE':
			return state.setIn(['entries'].concat(_toConsumableArray(action.keyPath)), action.value);
		case 'INCREMENT_VALUE':
			return state.updateIn(['entries'].concat(_toConsumableArray(action.keyPath)), function (oldValue) {
				return oldValue + 1;
			});
		case 'UPDATE_METHODS':
			return state.updateIn(['entries', action.key, 'methods'], function (methods) {
				return methods.mergeDeep(_immutable2.default.fromJS(action.value));
			});
		default:
			return state;
	}
};

exports.default = entries;


var initMethods = function initMethods() {
	var methodNames = ['constructorMethod', 'componentWillMount', 'componentDidMount', 'componentWillReceiveProps', 'shouldComponentUpdate', 'componentWillUpdate', 'render', 'componentDidUpdate', 'componentWillUnmount'];
	var logObj = {};
	methodNames.forEach(function (name) {
		logObj[name] = {
			name: name,
			isMethodOverridden: false,
			called: false,
			count: 0,
			isInfiniteLoop: false,
			props: [],
			state: []
		};
	});
	return logObj;
};

// const checkEqualityOfPropsAndState = (methodObj) => {
// 	const isStateChanged = methodObj.get('oldState') !== methodObj.get('newState');
// 	return methodObj.set('isStateChanged', isStateChanged);
// };

module.exports = exports['default'];